@@ -20,20 +20,37 @@ function(x,                       # sample (the data)
   nx <- length(x)
   if (is.null(bw)) bw <- bw.SJ(x)
   
-  fn <-
+  #fn <-
+  #function(z)
+  #{
+  #  z <- (rep(z,nx) - x)/rep(bw,nx)   #! enlever 'rep' : on ne peut pas appliquer 'fn'  un vecteur 'z' de longueur > 1 !
+  #                                    #! cela permettrait d'accelerer la methode 'abc'  
+  #  z <- do.call(paste(".kernel.d", kernel, sep = ""), list(z))$k
+  #  return(sum(z))   #!  modifier
+  #}
+
+  fn <- 
   function(z)
   {
-    z <- (rep(z,nx) - x)/rep(bw,nx)   #! enlever 'rep' : on ne peut pas appliquer 'fn'  un vecteur 'z' de longueur > 1 !
-                                      #! cela permettrait d'accelerer la methode 'abc'  
-    z <- do.call(paste(".kernel.d", kernel, sep = ""), list(z))$k
-    return(sum(z))   #!  modifier
+    mat <- z/bw - x/bw
+    k <- do.call(paste(".kernel.d", kernel, sep = ""), list(mat))$k
+    return(sum(k))
   }
-
+  
+  #FN <- 
+  #function(z)
+  #{
+  #  mat <- kronecker(z/bw, t(-x/bw), FUN = "+")
+  #  k <- do.call(paste(".kernel.d", kernel, sep = ""), list(mat))$k
+  #  return(rowSums(k))
+  #}
+  
   if (!abc) {
     r <- uniroot(f = fn, interval = c(min(x), max(x)), ...)
     M <- r$root
   } else {
-    f <- abs(sapply(x,FUN=fn)) #! pas tres rapide... mieux vaut dans ce cas utiliser 'density' je pense
+    FN <- Vectorize(fn)
+    f <- abs(FN(x)) 
     M <- x[f == min(f)]
   }
   
